@@ -29,18 +29,22 @@ static char rcsid[] = "$Id: signal.c 138 2006-09-22 22:12:03Z mikes@u.washington
 #include "../pith/status.h"
 #include "../pith/busy.h"
 #include "../pith/debug.h"
+#include "../pith/help.h"
+
+#include "../pith/charconv/utf8.h"
 
 #ifdef _WINDOWS
 #include "../pico/osdep/mswin.h"
 #endif
 
+#include "status.h"
 #include "after.h"
 #include "busy.h"
 
 
 static int       dotcount;
 static char      busy_message[MAX_BM + 1];
-static int       busy_cue_on;
+static int       busy_cue_state;
 static int       busy_cue_pause;
 static int       busy_width;
 static int       final_message;
@@ -122,14 +126,23 @@ busy_cue(char *msg, percent_done_t pc_f, int delay)
 
     dprint((9, "busy_cue(%s, %p, %d)\n", msg ? msg : "Busy", pc_f, delay));
 
-    if(!(ps_global && ps_global->ttyo))
-      return(0);
+    if(!(ps_global && ps_global->ttyo)){
+	dprint((9, "busy_cue returns No (ttyo)"));
+	return(0);
+    }
 
     /*
      * If we're already busy'ing, but a cue is invoked that
      * supplies more useful info, use it...
      */
-    if(busy_cue_on){
+    if(busy_cue_state == BUSY_CUE_DYING){
+	/* wait for the death to finish */
+	if(short_wait_for_busy_thread() != BUSY_CUE_OFF){
+	    dprint((9, "busy_cue returns No after waiting\n"));
+	    return(0);
+	}
+    }
+    else if(busy_cue_state == BUSY_CUE_ON){
 	if(msg || pc_f)
 	  stop_after(1);	/* uninstall old handler */
 	else
@@ -139,8 +152,6 @@ busy_cue(char *msg, percent_done_t pc_f, int delay)
     /* get ready to set up list of AFTER_S's */
     ap = &a;
 
-    busy_cue_on = 1;
-
     dotcount = 0;
     percent_done_ptr = pc_f;
 
@@ -250,6 +261,8 @@ busy_cue(char *msg, percent_done_t pc_f, int delay)
     (*ap)->cf	 = done_busy_cue;
     ap		 = &(*ap)->next;
 
+    busy_cue_state = BUSY_CUE_ON;
+
     start_after(a);		/* launch cue handler */
 
 #ifdef _WINDOWS
@@ -270,11 +283,11 @@ cancel_busy_cue(int message_pri)
 {
     dprint((9, "cancel_busy_cue(%d)\n", message_pri));
 
-    final_message_pri = message_pri;
-
-    stop_after(0);
-
-    busy_cue_on = 0;
+    if(busy_cue_state == BUSY_CUE_ON){
+	final_message_pri = message_pri;
+	busy_cue_state = BUSY_CUE_DYING;
+	stop_after(0);
+    }
 }
 
 /*
@@ -285,7 +298,7 @@ suspend_busy_cue(void)
 {
     dprint((9, "suspend_busy_cue\n"));
 
-    if(busy_cue_on)
+    if(busy_cue_state != BUSY_CUE_OFF)
       busy_cue_pause = 1;
 }
 
@@ -298,9 +311,8 @@ resume_busy_cue(unsigned int pause)
 {
     dprint((9, "resume_busy_cue\n"));
 
-    if(busy_cue_on){
+    if(busy_cue_state != BUSY_CUE_OFF)
       busy_cue_pause = 0;
-    }
 }
 
 
@@ -436,5 +448,49 @@ done_busy_cue(void *data)
     }
 
     mark_status_dirty();
+    busy_cue_state = BUSY_CUE_OFF;
 }
 
+
+/*
+ * We don't use these to access busy_cue_state from within this
+ * file, only from outside.
+ */
+int
+get_busy_cue_state(void)
+{
+    return(busy_cue_state);
+}
+
+
+int
+set_busy_cue_state(int newvalue)
+{
+    int oldvalue = busy_cue_state;
+
+    busy_cue_state = newvalue;
+    return(oldvalue);
+}
+
+
+int
+short_wait_for_busy_thread(void)
+{
+#if defined(HAVE_NANOSLEEP)
+    int i = 5;
+    struct timespec ts;
+
+    if(busy_cue_state == BUSY_CUE_DYING){
+	ts.tv_sec = 0;
+	ts.tv_nsec = 10000000;		/* 1/100 of a second */
+	(void) nanosleep(&ts, NULL);
+
+	ts.tv_nsec = 100000000;		/* 1/10 of a second */
+	while(busy_cue_state == BUSY_CUE_DYING && i-- > 0){
+	    (void) nanosleep(&ts, NULL);
+	}
+    }
+#endif
+
+    return(busy_cue_state);
+}
