@@ -1,5 +1,5 @@
 /*
- * $Id: setup.h 136 2006-09-22 20:06:05Z hubert@u.washington.edu $
+ * $Id: setup.h 569 2007-05-15 01:17:32Z mikes@u.washington.edu $
  *
  * ========================================================================
  * Copyright 2006 University of Washington
@@ -24,6 +24,8 @@
 /* exported protoypes */
 void	    option_screen(struct pine *, int);
 int         litsig_text_tool(struct pine *, int, CONF_S **, unsigned);
+char	   *pretty_var_name(char *);
+char	   *pretty_feature_name(char *);
 
 
 #endif /* PINE_SETUP_INCLUDED */
