@@ -23,7 +23,7 @@
  *		Internet: MRC@CAC.Washington.EDU
  *
  * Date:	11 June 1997
- * Last Edited:	25 May 2007
+ * Last Edited:	29 August 2007
  */
 
 
@@ -901,7 +901,7 @@ unsigned long utf8_get (unsigned char **s,unsigned long *i)
 
 unsigned long utf8_get_raw (unsigned char **s,unsigned long *i)
 {
-  unsigned char c;
+  unsigned char c,c1;
   unsigned char *t = *s;
   unsigned long j = *i;
   unsigned long ret = U8G_NOTUTF8;
@@ -919,22 +919,23 @@ unsigned long utf8_get_raw (unsigned char **s,unsigned long *i)
 				/* incomplete UTF-8 character */
     else if (more) return U8G_INCMPLT;
     else {			/* start of sequence */
+      c1 = j ? *t : 0xbf;	/* assume valid continuation if incomplete */
       if (c < 0x80) ret = c;	/* U+0000 - U+007f */
       else if (c < 0xc2);	/* c0 and c1 never valid */
       else if (c < 0xe0) {	/* U+0080 - U+07ff */
 	if (c &= 0x1f) more = 1;
       }
       else if (c < 0xf0) {	/* U+0800 - U+ffff */
-	if ((c &= 0x0f) || (*t >= 0xa0)) more = 2;
+	if ((c &= 0x0f) || (c1 >= 0xa0)) more = 2;
       }
       else if (c < 0xf8) {	/* U+10000 - U+10ffff (and 110000 - 1fffff) */
-	if ((c &= 0x07) || (*t >= 0x90)) more = 3;
+	if ((c &= 0x07) || (c1 >= 0x90)) more = 3;
       }
       else if (c < 0xfc) {	/* ISO 10646 200000 - 3ffffff */
-	if ((c &= 0x03) || (*t >= 0x88)) more = 4;
+	if ((c &= 0x03) || (c1 >= 0x88)) more = 4;
       }
       else if (c < 0xfe) {	/* ISO 10646 4000000 - 7fffffff */
-	if ((c &= 0x01) || (*t >= 0x84)) more = 5;
+	if ((c &= 0x01) || (c1 >= 0x84)) more = 5;
       }
 				/* fe and ff never valid */
       if (more) {		/* multi-octet, make sure more to come */
