@@ -23,7 +23,7 @@
  *		Internet: MRC@CAC.Washington.EDU
  *
  * Date:	20 December 1989
- * Last Edited:	18 June 2007
+ * Last Edited:	30 August 2007
  */
 
 
@@ -162,7 +162,7 @@ long mbox_append (MAILSTREAM *stream,char *mailbox,append_t af,void *data);
 DRIVER unixdriver = {
   "unix",			/* driver name */
 				/* driver flags */
-  DR_LOCAL|DR_MAIL|DR_LOCKING|DR_NONEWMAILRONLY,
+  DR_LOCAL|DR_MAIL|DR_LOCKING|DR_NONEWMAILRONLY|DR_XPOINT,
   (DRIVER *) NIL,		/* next driver */
   unix_valid,			/* mailbox is valid for us */
   unix_parameters,		/* manipulate parameters */
@@ -633,10 +633,10 @@ char *unix_header (MAILSTREAM *stream,unsigned long msgno,
 				/* got text, tie off string */
     LOCAL->buf[*length = elt->private.msg.header.text.size] = '\0';
 				/* squeeze out CRs (in case from PC) */
-    for (s = t = LOCAL->buf,tl = LOCAL->buf + *length; t <= tl; t++)
-      if ((*t != '\r') || (t[1] != '\n')) *s++ = *t;
-				/* adjust length */
-    LOCAL->buf[*length = s - LOCAL->buf - 1] = '\0';
+    for (s = t = LOCAL->buf,tl = LOCAL->buf + *length; t < tl; t++)
+      if (*t != '\r') *s++ = *t;
+    *s = '\0';
+    *length = s - LOCAL->buf;	/* adjust length */
   }
   else {			/* need to make a CRLF version */
     read (LOCAL->fd,s = (char *) fs_get (elt->private.msg.header.text.size+1),
@@ -646,6 +646,11 @@ char *unix_header (MAILSTREAM *stream,unsigned long msgno,
     *length = strcrlfcpy (&LOCAL->buf,&LOCAL->buflen,s,
 			  elt->private.msg.header.text.size);
     fs_give ((void **) &s);	/* free readin buffer */
+				/* squeeze out spurious CRs */
+    for (s = t = LOCAL->buf,tl = LOCAL->buf + *length; t < tl; t++)
+      if ((*t != '\r') || (t[1] == '\n')) *s++ = *t;
+    *s = '\0';
+    *length = s - LOCAL->buf;	/* adjust length */
   }
   *length = mail_filter (LOCAL->buf,*length,unix_hlines,FT_NOT);
   return (char *) LOCAL->buf;	/* return processed copy */
@@ -705,10 +710,10 @@ char *unix_text_work (MAILSTREAM *stream,MESSAGECACHE *elt,
 				/* got text, tie off string */
     LOCAL->buf[*length = elt->private.msg.text.text.size] = '\0';
 				/* squeeze out CRs (in case from PC) */
-    for (s = t = LOCAL->buf,tl = LOCAL->buf + *length; t <= tl; t++)
-      if ((*t != '\r') || (t[1] != '\n')) *s++ = *t;
-				/* adjust length */
-    LOCAL->buf[*length = s - LOCAL->buf - 1] = '\0';
+    for (s = t = LOCAL->buf,tl = LOCAL->buf + *length; t < tl; t++)
+      if (*t != '\r') *s++ = *t;
+    *s = '\0';
+    *length = s - LOCAL->buf;	/* adjust length */
     return (char *) LOCAL->buf;
   }
 
@@ -730,8 +735,6 @@ char *unix_text_work (MAILSTREAM *stream,MESSAGECACHE *elt,
     INIT (&bs,fd_string,&d,elt->private.msg.text.text.size);
     for (s = (char *) LOCAL->text.data; SIZE (&bs);) switch (CHR (&bs)) {
     case '\r':			/* carriage return seen */
-      *s++ = SNX (&bs);		/* copy it and any succeeding LF */
-      if (SIZE (&bs) && (CHR (&bs) == '\n')) *s++ = SNX (&bs);
       break;
     case '\n':
       *s++ = '\r';		/* insert a CR */
@@ -901,43 +904,42 @@ long unix_copy (MAILSTREAM *stream,char *sequence,char *mailbox,long options)
   if (!((options & CP_UID) ? mail_uid_sequence (stream,sequence) :
 	mail_sequence (stream,sequence))) return NIL;
 				/* make sure destination is valid */
-  if ((d && mbox_valid (mailbox) && (mailbox = "mbox")) ||
-      unix_valid (mailbox) || !errno) {
-				/* try to open rewrite or at least sniff */
-    if (!(tstream = mail_open_work (&unixdriver,NIL,mailbox,
-				    OP_SILENT|OP_NOKOD)) &&
-	!(tstream = mail_open_work (&unixdriver,NIL,mailbox,
-				    OP_READONLY|OP_SILENT|OP_NOKOD|OP_SNIFF))){
-      sprintf (LOCAL->buf,"Unable to open mailbox for COPY: %.80s",mailbox);
+  if (!((d && mbox_valid (mailbox) && (mailbox = "mbox")) ||
+	unix_valid (mailbox) || !errno))
+    switch (errno) {
+    case ENOENT:		/* no such file? */
+      if (compare_cstring (mailbox,"INBOX")) {
+	MM_NOTIFY (stream,"[TRYCREATE] Must create mailbox before copy",NIL);
+	return NIL;
+      }
+      if (pc) return (*pc) (stream,sequence,mailbox,options);
+      unix_create (NIL,"INBOX");/* create empty INBOX */
+    case EACCES:		/* file protected */
+      sprintf (LOCAL->buf,"Can't access destination: %.80s",mailbox);
+      MM_LOG (LOCAL->buf,ERROR);
+      return NIL;
+    case EINVAL:
+      if (pc) return (*pc) (stream,sequence,mailbox,options);
+      sprintf (LOCAL->buf,"Invalid UNIX-format mailbox name: %.80s",mailbox);
+      MM_LOG (LOCAL->buf,ERROR);
+      return NIL;
+    default:
+      if (pc) return (*pc) (stream,sequence,mailbox,options);
+      sprintf (LOCAL->buf,"Not a UNIX-format mailbox: %.80s",mailbox);
       MM_LOG (LOCAL->buf,ERROR);
       return NIL;
     }
-  }
-  else switch (errno) {
-  case ENOENT:			/* no such file? */
-    if (compare_cstring (mailbox,"INBOX")) {
-      MM_NOTIFY (stream,"[TRYCREATE] Must create mailbox before copy",NIL);
-      return NIL;
-    }
-    if (pc) return (*pc) (stream,sequence,mailbox,options);
-    unix_create (NIL,"INBOX");	/* create empty INBOX */
-  case EACCES:			/* file protected */
-    sprintf (LOCAL->buf,"Can't access destination: %.80s",mailbox);
-    MM_LOG (LOCAL->buf,ERROR);
-    return NIL;
-  case EINVAL:
-    if (pc) return (*pc) (stream,sequence,mailbox,options);
-    sprintf (LOCAL->buf,"Invalid UNIX-format mailbox name: %.80s",mailbox);
-    MM_LOG (LOCAL->buf,ERROR);
-    return NIL;
-  default:
-    if (pc) return (*pc) (stream,sequence,mailbox,options);
-    sprintf (LOCAL->buf,"Not a UNIX-format mailbox: %.80s",mailbox);
-    MM_LOG (LOCAL->buf,ERROR);
-    return NIL;
-  }
 
-  if (tstream->rdonly) cu = NIL;/* don't COPYUID if can't update uid_last */
+				/* try to open rewrite for UIDPLUS */
+  if ((tstream = mail_open_work (&unixdriver,NIL,mailbox,
+				 OP_SILENT|OP_NOKOD)) && tstream->rdonly)
+    tstream = mail_close (tstream);
+  if (cu && !tstream) {		/* wanted a COPYUID? */
+    sprintf (LOCAL->buf,"Unable to write-open mailbox for COPYUID: %.80s",
+	     mailbox);
+    MM_LOG (LOCAL->buf,WARN);
+    cu = NIL;			/* don't try to do COPYUID */
+  }
   LOCAL->buf[0] = '\0';
   MM_CRITICAL (stream);		/* go critical */
   if ((fd = unix_lock (dummy_file (file,mailbox),O_WRONLY|O_APPEND,
@@ -961,8 +963,9 @@ long unix_copy (MAILSTREAM *stream,char *sequence,char *mailbox,long options)
 	if (j && (s[j - 2] == '\n')) j--;
 	if (write (fd,s,j) < 0) ret = NIL;
 	else {			/* message header succeeded */
-	  j = unix_xstatus (stream,LOCAL->buf,elt,
-			    cu ? ++(tstream->uid_last) : NIL,cu ? LONGT : NIL);
+	  j = tstream ?		/* write UIDPLUS data if have readwrite */
+	    unix_xstatus (stream,LOCAL->buf,elt,++(tstream->uid_last),LONGT) :
+	    unix_xstatus (stream,LOCAL->buf,elt,NIL,NIL);
 	  if (write (fd,LOCAL->buf,j) < 0) ret = NIL;
 	  else {		/* message status succeeded */
 	    s = unix_text_work (stream,elt,&j,FT_INTERNAL);
@@ -982,11 +985,9 @@ long unix_copy (MAILSTREAM *stream,char *sequence,char *mailbox,long options)
     ret = NIL;
   }
 				/* force UIDVALIDITY assignment now */
-  if (!tstream->uid_validity && !tstream->rdonly)
-    tstream->uid_validity = time (0);
+  if (tstream && !tstream->uid_validity) tstream->uid_validity = time (0);
 				/* return sets if doing COPYUID */
-  if (cu && ret && tstream->uid_validity)
-    (*cu) (stream,mailbox,tstream->uid_validity,source,dest);
+  if (cu && ret) (*cu) (stream,mailbox,tstream->uid_validity,source,dest);
   else {			/* flush any sets we may have built */
     mail_free_searchset (&source);
     mail_free_searchset (&dest);
@@ -998,12 +999,12 @@ long unix_copy (MAILSTREAM *stream,char *sequence,char *mailbox,long options)
 	 sbuf.st_atime : tp[1];
   utime (file,tp);		/* set the times */
   unix_unlock (fd,NIL,&lock);	/* unlock and close mailbox */
-  if (!tstream->rdonly) {	/* update last UID if we can */
+  if (tstream) {		/* update last UID if we can */
     UNIXLOCAL *local = (UNIXLOCAL *) tstream->local;
     local->dirty = T;		/* do a rewrite */
     local->appending = T;	/* but not at the cost of marking as old */
+    tstream = mail_close (tstream);
   }
-  tstream = mail_close (tstream);
 				/* log the error */
   if (!ret) MM_LOG (LOCAL->buf,ERROR);
 				/* delete if requested message */
@@ -1046,18 +1047,8 @@ long unix_append (MAILSTREAM *stream,char *mailbox,append_t af,void *data)
     stream = &unixproto;	/* no, default stream to prototype */
     for (i = 0; i < NUSERFLAGS && stream->user_flags[i]; ++i)
       fs_give ((void **) &stream->user_flags[i]);
-    stream->kwd_create = T;	/* allow new flags */
   }
-  if (unix_valid (mailbox)) {	/* make sure valid mailbox */
-    if (!(tstream = mail_open (NIL,mailbox,
-			       OP_READONLY|OP_SILENT|OP_NOKOD|OP_SNIFF))) {
-      sprintf (tmp,"Unable to examine mailbox for APPEND: %.80s",mailbox);
-      MM_LOG (tmp,ERROR);
-      return NIL;
-    }
-  }
-
-  else switch (errno) {
+  if (!unix_valid (mailbox)) switch (errno) {
   case ENOENT:			/* no such file? */
     if (compare_cstring (mailbox,"INBOX")) {
       MM_NOTIFY (stream,"[TRYCREATE] Must create mailbox before append",NIL);
@@ -1080,6 +1071,14 @@ long unix_append (MAILSTREAM *stream,char *mailbox,append_t af,void *data)
     MM_LOG (tmp,ERROR);
     return NIL;
   }
+				/* get sniffing stream for keywords */
+  else if (!(tstream = mail_open (NIL,mailbox,
+				  OP_READONLY|OP_SILENT|OP_NOKOD|OP_SNIFF))) {
+    sprintf (tmp,"Unable to examine mailbox for APPEND: %.80s",mailbox);
+    MM_LOG (tmp,ERROR);
+    return NIL;
+  }
+
 				/* get first message */
   if (!MM_APPEND (af) (tstream,data,&flags,&date,&message)) return NIL;
   if (!(sf = tmpfile ())) {	/* must have scratch file */
@@ -1091,7 +1090,6 @@ long unix_append (MAILSTREAM *stream,char *mailbox,append_t af,void *data)
     }
     unlink (tmp);
   }
-
   do {				/* parse date */
     if (!date) rfc822_date (date = tmp);
     if (!mail_parse_date (&elt,date)) {
@@ -1116,27 +1114,26 @@ long unix_append (MAILSTREAM *stream,char *mailbox,append_t af,void *data)
     fclose (sf);		/* punt scratch file */
     return NIL;			/* give up */
   } while (message);		/* until no more messages */
-  if (fflush (sf) || fstat (fileno (sf),&sbuf)) {
+  if (fflush (sf)) {
     sprintf (tmp,"Error finishing scratch file: %.80s",strerror (errno));
     MM_LOG (tmp,ERROR);
     fclose (sf);		/* punt scratch file */
     return NIL;			/* give up */
   }
-  i = sbuf.st_size;		/* size of scratch file */
-
+  i = ftell (sf);		/* size of scratch file */
 				/* close sniffing stream */
   if (tstream != stream) tstream = mail_close (tstream);
+
   MM_CRITICAL (stream);		/* go critical */
-				/* try to open readwrite */
-  if (!(tstream = mail_open_work (&unixdriver,NIL,mailbox,
-				  OP_SILENT|OP_NOKOD)) &&
-      !(tstream = mail_open_work (&unixdriver,NIL,mailbox,
-				  OP_READONLY|OP_SILENT|OP_NOKOD|OP_SNIFF))) {
-    sprintf (tmp,"Unable to re-open mailbox for APPEND: %.80s",mailbox);
-    MM_LOG (tmp,ERROR);
-    return NIL;
+				/* try to open readwrite for UIDPLUS */
+  if ((tstream = mail_open_work (&unixdriver,NIL,mailbox,
+				 OP_SILENT|OP_NOKOD)) && tstream->rdonly)
+    tstream = mail_close (tstream);
+  if (au && !tstream) {		/* wanted an APPENDUID? */
+    sprintf (tmp,"Unable to re-open mailbox for APPENDUID: %.80s",mailbox);
+    MM_LOG (tmp,WARN);
+    au = NIL;
   }
-  if (tstream->rdonly) au = NIL;/* don't APPENDUID if can't update uid_last */
   if (((fd = unix_lock (dummy_file (file,mailbox),O_WRONLY|O_APPEND,
 		       (long) mail_parameters (NIL,GET_MBXPROTECTION,NIL),
 			&lock,LOCK_EX)) < 0) ||
@@ -1164,20 +1161,18 @@ long unix_append (MAILSTREAM *stream,char *mailbox,append_t af,void *data)
   utime (file,tp);		/* set the times */
   fclose (sf);			/* done with scratch file */
 				/* force UIDVALIDITY assignment now */
-  if (!tstream->uid_validity && !tstream->rdonly)
-    tstream->uid_validity = time (0);
+  if (tstream && !tstream->uid_validity) tstream->uid_validity = time (0);
 				/* return sets if doing APPENDUID */
-  if (au && ret && tstream->uid_validity)
-    (*au) (mailbox,tstream->uid_validity,dst);
+  if (au && ret) (*au) (mailbox,tstream->uid_validity,dst);
   else mail_free_searchset (&dst);
   unix_unlock (fd,NIL,&lock);	/* unlock and close mailbox */
   fclose (df);			/* note that unix_unlock() released the fd */
-  if (!tstream->rdonly) {	/* update last UID if we can */
+  if (tstream) {		/* update last UID if we can */
     UNIXLOCAL *local = (UNIXLOCAL *) tstream->local;
     local->dirty = T;		/* do a rewrite */
     local->appending = T;	/* but not at the cost of marking as old */
+    tstream = mail_close (tstream);
   }
-  tstream = mail_close (tstream);
   MM_NOCRITICAL (stream);	/* release critical */
   return ret;
 }
@@ -1198,7 +1193,11 @@ int unix_collect_msg (MAILSTREAM *stream,FILE *sf,char *flags,char *date,
   unsigned long uf;
   long f = mail_parse_flags (stream,flags,&uf);
 				/* write metadata, note date ends with NL */
-  if (fprintf (sf,"%ld %lu %lu %s",f,uf,SIZE (msg) + 1,date) < 0) return NIL;
+  if (fprintf (sf,"%ld %lu %s",f,SIZE (msg) + 1,date) < 0) return NIL;
+  while (uf)			/* write user flags */    
+    if ((s = stream->user_flags[find_rightmost_bit (&uf)]) &&
+	(fprintf (sf," %s",s) < 0)) return NIL;
+  if (putc ('\n',sf) == EOF) return NIL;
   while (SIZE (msg)) {		/* copy text to scratch file */
     for (s = (unsigned char *) msg->curpos, t = s + msg->cursize; s < t; ++s)
       if (!*s) *s = 0x80;	/* disallow NUL */
@@ -1223,7 +1222,7 @@ int unix_append_msgs (MAILSTREAM *stream,FILE *sf,FILE *df,SEARCHSET *set)
 {
   int ti,zn,c;
   long f;
-  unsigned long i,j,uf;
+  unsigned long i,j;
   char *x,tmp[MAILTMPLEN];
   int hdrp = T;
 				/* get message metadata line */
@@ -1231,8 +1230,6 @@ int unix_append_msgs (MAILSTREAM *stream,FILE *sf,FILE *df,SEARCHSET *set)
     if (!(isdigit (tmp[0]) && strchr (tmp,'\n'))) return NIL;
     f = strtol (tmp,&x,10);	/* get flags */
     if (!((*x++ == ' ') && isdigit (*x))) return NIL;
-    uf = strtoul (x,&x,10);	/* get keywords */
-    if (!((*x++ == ' ') && isdigit (*x))) return NIL;
     i = strtoul (x,&x,10);	/* get message size */
     if ((*x++ != ' ') ||	/* build initial header */
 	(fprintf (df,"From %s@%s %sStatus: ",myusername(),mylocalhost(),x)<0)||
@@ -1243,13 +1240,16 @@ int unix_append_msgs (MAILSTREAM *stream,FILE *sf,FILE *df,SEARCHSET *set)
 	(f&fANSWERED && (putc ('A',df) == EOF)) ||
 	(f&fDRAFT && (putc ('T',df) == EOF)) ||
 	(fputs ("\nX-Keywords:",df) == EOF)) return NIL;
-    while (uf)			/* write user flags */
-      if (fprintf (df," %s",stream->user_flags[find_rightmost_bit (&uf)]) < 0)
-	return NIL;
-    if (set && (fprintf (df,"\nX-UID: %lu",++(stream->uid_last)) < 0))
+				/* copy keywords */
+    while ((c = getc (sf)) != '\n') switch (c) {
+    case EOF:
+      return NIL;
+    default:
+      if (putc (c,df) == EOF) return NIL;
+    }
+    if ((putc ('\n',df) == EOF) ||
+	(set && (fprintf (df,"X-UID: %lu\n",++(stream->uid_last)) < 0)))
       return NIL;
-				/* finish last metadata header */
-    if (putc ('\n',df) == EOF) return NIL;
 
     for (c = '\n'; i && fgets (tmp,MAILTMPLEN,sf); c = tmp[j-1]) {
 				/* get read line length */
@@ -1323,7 +1323,7 @@ int unix_append_msgs (MAILSTREAM *stream,FILE *sf,FILE *df,SEARCHSET *set)
     }
     if (i) return NIL;		/* didn't read entire message */
 				/* update set */
-    mail_append_set (set,stream->uid_last);
+    if (stream) mail_append_set (set,stream->uid_last);
   }
   return T;
 }
@@ -1736,7 +1736,7 @@ int unix_parse (MAILSTREAM *stream,DOTLOCK *lock,int op)
 	      for (u = s,v = tmp,e = u + min (i,MAILTMPLEN - 1);
 		   (u < e) && ((c = (*u ? *u : (*u = ' '))) != ':') &&
 		   ((c > ' ') || ((c != ' ') && (c != '\t') &&
-				  (c != '\015') && (c != '\012')));
+				  (c != '\r') && (c != '\n')));
 		   *v++ = *u++);
 	      *v = '\0';	/* tie off */
 				/* matches internal header? */
@@ -1758,7 +1758,7 @@ int unix_parse (MAILSTREAM *stream,DOTLOCK *lock,int op)
 	    if (retain || ((*s != ' ') && (*s != '\t'))) {
 	      retain = T;	/* retaining continuation now */
 				/* line length in LF format newline */
-	      k = i - (((i >= 2) && (s[i - 2] == '\r')) ? 1 : 0);
+	      for (j = k = 0; j < i; ++j) if (s[j] != '\r') ++k;
 				/* "internal" header size */
 	      elt->private.spare.data += k;
 				/* message size */
@@ -1798,13 +1798,24 @@ int unix_parse (MAILSTREAM *stream,DOTLOCK *lock,int op)
 	  if (i) {		/* got new data? */
 	    VALID (s,t,ti,zn);	/* yes, parse line */
 	    if (!ti) {		/* not a header line, add it to message */
-	      if (s[i - 1] == '\n')
-		elt->rfc822_size += 
-		  k = i + (m = (((i < 2) || s[i - 2] != '\r') ? 1 : 0));
-	      else {		/* file does not end with newline! */
-		elt->rfc822_size += i;
-		k = m = 0;
+	      elt->rfc822_size += i;
+	      for (j = 0; j < i; ++j) switch (s[j]) {
+	      case '\r':	/* squeeze out CRs */
+		elt->rfc822_size -= 1;
+		break;
+	      case '\n':	/* LF becomes CRLF */
+		elt->rfc822_size += 1;
+		break;
+	      default:
+		break;
 	      }
+	      if ((i == 1) && (*s == '\n')) {
+		k = 2;
+		m = 1;
+	      }
+	      else if ((i == 2) && (*s == '\r') && (s[1] == '\n'))
+		k = m = 2;
+	      else k = m = 0;	/* file does not end with newline! */
 				/* update current position */
 	      j = LOCAL->filesize + GETPOS (&bs);
 	    }
@@ -1812,10 +1823,9 @@ int unix_parse (MAILSTREAM *stream,DOTLOCK *lock,int op)
 	} while (i && !ti);	/* until found a header */
 	elt->private.msg.text.text.size = j -
 	  (elt->private.special.offset + elt->private.msg.text.offset);
-	if (k == 2) {		/* last line was blank? */
-	  elt->private.msg.text.text.size -= (m ? 1 : 2);
-	  elt->rfc822_size -= 2;
-	}
+				/* flush ending blank line */
+	elt->private.msg.text.text.size -= m;
+	elt->rfc822_size -= k;
 				/* until end of buffer */
       } while (!stream->sniff && i);
       if (pseudoseen) {		/* flush pseudo-message if present */
@@ -2144,7 +2154,14 @@ long unix_rewrite (MAILSTREAM *stream,unsigned long *nexp,DOTLOCK *lock,
 	  elt->private.msg.header.offset = elt->private.special.text.size;
 				/* header size, sans trailing newline */
 	  if ((j < 2) || (s[j - 2] == '\n')) j--;
-	  if (j != elt->private.spare.data) fatal ("header size inconsistent");
+				/* this can happen if CRs were squeezed */
+	  if (j < elt->private.spare.data) {
+				/* so fix up counts */
+	    size -= elt->private.spare.data - j;
+	    elt->private.spare.data = j;
+	  }
+	  else if (j != elt->private.spare.data)
+	    fatal ("header size inconsistent");
 				/* protection pointer moves to RFC822 text */
 	  f.protect = elt->private.special.offset +
 	    elt->private.msg.text.offset;
@@ -2577,7 +2594,8 @@ long mbox_ping (MAILSTREAM *stream)
       (time (0) >= (LOCAL->lastsnarf +
 		    (long) mail_parameters (NIL,GET_SNARFINTERVAL,NIL))) &&
       !stat (sysinbox (),&sbuf) && sbuf.st_size) {
-				/* yes, open and lock sysinbox */
+    MM_CRITICAL (stream);	/* yes, go critical */
+				/* open and lock sysinbox */
     if ((sfd = unix_lock (sysinbox (),O_RDWR,
 			  (long) mail_parameters (NIL,GET_MBXPROTECTION,NIL),
 			  &lockx,LOCK_EX)) >= 0) {
@@ -2636,12 +2654,12 @@ long mbox_ping (MAILSTREAM *stream)
 				/* all done with mbox */
 	unix_unlock (LOCAL->fd,stream,&lock);
 	mail_unlock (stream);	/* unlock the stream */
-				/* done with critical */
-	MM_NOCRITICAL (stream);
+	MM_NOCRITICAL (stream);	/* done with critical */
       }
 				/* all done with sysinbox */
       unix_unlock (sfd,NIL,&lockx);
     }
+    MM_NOCRITICAL (stream);	/* done with critical */
     LOCAL->lastsnarf = time (0);/* note time of last snarf */
   }
   return unix_ping (stream);	/* do the unix routine now */
