@@ -1,5 +1,5 @@
 /*
- * $Id: conftype.h 607 2007-06-22 17:47:59Z hubert@u.washington.edu $
+ * $Id: conftype.h 660 2007-07-31 20:49:58Z mikes@u.washington.edu $
  *
  * ========================================================================
  * Copyright 2006-2007 University of Washington
@@ -215,6 +215,8 @@ typedef	enum {    V_PERSONAL_NAME = 0
 		, V_QUOTE2_BACK_COLOR
 		, V_QUOTE3_FORE_COLOR
 		, V_QUOTE3_BACK_COLOR
+		, V_INCUNSEEN_FORE_COLOR
+		, V_INCUNSEEN_BACK_COLOR
 		, V_SIGNATURE_FORE_COLOR
 		, V_SIGNATURE_BACK_COLOR
 		, V_PROMPT_FORE_COLOR
@@ -233,8 +235,14 @@ typedef	enum {    V_PERSONAL_NAME = 0
 		, V_IND_NEW_BACK_COLOR
 		, V_IND_REC_FORE_COLOR
 		, V_IND_REC_BACK_COLOR
+		, V_IND_FWD_FORE_COLOR
+		, V_IND_FWD_BACK_COLOR
 		, V_IND_UNS_FORE_COLOR
 		, V_IND_UNS_BACK_COLOR
+		, V_IND_HIPRI_FORE_COLOR
+		, V_IND_HIPRI_BACK_COLOR
+		, V_IND_LOPRI_FORE_COLOR
+		, V_IND_LOPRI_BACK_COLOR
 		, V_IND_ARR_FORE_COLOR
 		, V_IND_ARR_BACK_COLOR
 		, V_IND_SUBJ_FORE_COLOR
@@ -445,6 +453,7 @@ typedef enum {
 	F_SHOW_TEXTPLAIN_INT,
 	F_ROLE_CONFIRM_DEFAULT,
 	F_TAB_NO_CONFIRM,
+	F_DATES_TO_LOCAL,
 	F_RET_INBOX_NO_CONFIRM,
 	F_CHECK_MAIL_ONQUIT,
 	F_PREOPEN_STAYOPENS,
@@ -485,6 +494,8 @@ typedef enum {
 	F_ENABLE_JUMP_CMD,
 	F_USE_SYSTEM_TRANS,
 	F_QUELL_HOST_AFTER_URL,
+	F_NNTP_SEARCH_USES_OVERVIEW,
+	F_THREAD_SORTS_BY_ARRIVAL,
 #ifdef	_WINDOWS
 	F_ENABLE_TRAYICON,
 	F_QUELL_SSL_LARGEBLOCKS,
